
(* Package for solving problems from Praktikum i/ii (physics) of the Technische Universitt Mnchen
  (C) 1997 by Walter Winter   Version 1.2 *)
   
BeginPackage["Statistics`Praktikum`"]

Needs["Statistics`LinearRegression`"]
Needs["Graphics`Spline`"]

SRound::usage = "SRound[{var,val,u}] rounds a mean value result of a measurement variable var (value val) with the estimated or calculated
 insafety u in relation to the rules fixed in the Praktikum i/ii skriptum from SS 1997. Therefore u is rounded to
one significant digit (except 1 or 2 -> two sign. digits) to its ceiling, whereas val is rounded to the precision of
u. The output is a list of the rounded values"

EvaluateSeries::usage = "EvaluateSeries[var,data,options] evaluates a series of data values from a measurement. The output is
a list of the form {var,mean,u}. The insafety u is calculated by means of the standard deviation of the mean and a
special confidence factor t, which is calculated at the niveau of 68,3%"

Gauss::usage = "GaussOR[expr,var,list,options] calculates the new insafety of an indirect measured size, which is calculated 
by the formula expr, whereas var is the name of the result variable. List is a list which contains lists of the form 
{var,mean,u}, where all parameters included in expr must be introduced with their means and insafeties. The result
of the function is a list {var,mean,u}, where var is the result parameter"

Rounding::usage = "Rounding is an option for Gauss and EvaluateSeries, which is initially True. In this case all results are
rounded by SRound"

Report::usage = "Report is an option for Gauss and EvaluateSeries, which is initially True. In this case the calculation process
is reported"

Regression::usage = "Regression[data,funs,var,param] implements a simplified linear regression function, which gets the
same parameters like Fit. The output form is prepared to be evaluated with Gauss: { function, variable, list of parameters}.
The difference to Fit is, that the function is given in a more general form with constant parameters generated
from param (param[1],param[2],...) plus the insafeties of the parameters in extra lists."

PolarListPlot::usage = "PolarListPlot[data,opts] implements a polar plot based on a list of data points (or multiple lists of data 
points)"

MaxValue::usage = "MaxValue is an option for PolarListPlot, which defines the maximum value (initially the maximum value of the data)"

ValueGap::usage = "ValueGap is an option for PolarListPlot, which defines the gap between the values (initially maximum value divided by 5)"

AngleGap::usage = "AngleGap is an option for PolarListPlot, which defines the gap between the angle lines (initially 30 degree)"

SplineInterpolation::usage = "SplineInterpolation is an option for PolarListPlot, which declares, if spline interpolations should be drawn, to
show the connections between the data points"

		
Begin["Statistics`Praktikum`Private`"]

SRound[list_List] := Module[{val = list[[2]], u = list[[3]],me,ures,valme,valres},
		me = MantissaExponent[N[u]];
  	If[me[[1]]>=10,me[[1]] = me[[1]]/10;me[[2]] = me[[2]] +1,
  	If[me[[1]]<1,me[[1]] = me[[1]]*10;me[[2]] = me[[2]] -1,]];
		If[IntegerPart[me[[1]]]<=2,me[[1]]=me[[1]]*10;me[[2]]=me[[2]]-1,];
		ures = N[Ceiling[me[[1]]]*10^me[[2]]];
		valme = N[{val/10^me[[2]],me[[2]]}];
		valres = N[IntegerPart[valme[[1]]+0.5]*10^valme[[2]]];
		{list[[1]],valres,ures}]

confintervalls =
   {2 -> 1.84, 3 -> 1.32, 4 -> 1.20, 5 -> 1.15, 6 -> 1.11, 7 -> 1.09, 
    8 -> 1.08, 9 -> 1.07, 10 -> 1.06}

GetConfidence[an_] := Module[{n=an,res},
		If[n<=10,res = n /. confintervalls,
		If[n<15, res = 1.05,
		If[n<17, res = 1.04,
		If[n<25, res = 1.03,
		If[n<40, res = 1.02,
		If[n<75, res = 1.01,1.00]]]]]];res]

EvaluateSeries[avar_,adata_List, aopts___Rule] := 
  Module[{var=avar,data=adata,mean,stddev,cf,u,report,rounding,opts = aopts, 
      sr},
		
		report = Report /. {opts} /. Report -> True ;
		rounding = Rounding /.{opts} /.  Rounding -> True;
		
		mean = Apply[Plus,data]/Length[data];
		stddev = Sqrt[Apply[Plus,Map[((#-mean)^2)&,data]]/(Length[data]-1)];
		cf = GetConfidence[Length[data]];
		u = cf/Sqrt[Length[data]]*stddev;
		If[rounding===True,sr = SRound[{var,mean,u}],sr={var, mean, u}];
		If[report===True,
			     Print["Anzahl Mewerte: ",Length[data]];
			     Print["Mittelwert: ",N[mean]];
              		     Print["Standardabweichung der Einzelmessung: ",N[stddev]];
			     Print["Standardabweichung des Mittelwerts: ",
                                                      N[stddev/Sqrt[Length[data]]]];
			     Print["t-Faktor (Vertrauensniveau 68,26%): ",cf];
			     Print["Meunsicherheit: ",N[u]];
		    	  If[rounding===True,
                            Print["Runden der Ergebnisse:  ",DisplayForm[SubscriptBox["u",var]],
                           " = ",N[u]," \[TildeTilde] ",sr[[3]],"  \[DoubleRightArrow]  ",
                           DisplayForm[OverBar[var]]," = ",N[mean]," \[TildeTilde] ",sr[[2]]]  ,];  ,];	N[sr]]

Gauss[aexpr_,avar_,alist_List, aopts___Rule] := 
  Module[{var=avar,expr = aexpr,list=alist, rules, repl, u, vls, ins, displ, 
      resdis, report, rounding, opts = aopts, res, sr},

		report = Report /. {opts} /. Report -> True ;
		rounding = Rounding /.{opts} /.  Rounding -> True;
		
		repl = Map[(D[expr,#[[1]]])&,list];
		rules = Map[(#[[1]] -> #[[2]])&,list];

		If[report===True,
		Print[Apply[Sequence,
                Flatten[Map[{#[[1]]," = ",#[[2]],"  ",#[[3]],"     "}&,list]]]];
		Print[var," = ",expr," = ",expr /. rules];
		vls = Map[#[[1]]&,list];
		ins = Map[DisplayForm[SubscriptBox["u",ToString[#]]]&,vls];
		displ = 
                 Map[(DisplayForm[
                 FractionBox[
                 RowBox[{"\[Delta]",StyleBox[ToString[var],FontSlant -> Italic]}],
		 RowBox[{"\[Delta]",StyleBox[ToString[#],FontSlant -> Italic]}]]])&,vls];

		resdis = DisplayForm[SubscriptBox["u",var]];
		Print[resdis," = ",Sqrt[Apply[Plus,Map[(#[[1]]^2*#[[2]]^2)&,Transpose[{ins,displ}]]]]];
		Print["    = ",Sqrt[Apply[Plus,Map[(#[[1]]^2*#[[2]]^2)&,Transpose[{ins,repl}]]]]],];

		u = Sqrt[Apply[Plus,Map[(#[[1]]^2*#[[2]]^2)&,Transpose[{Map[(#[[3]])&,list],repl}]]]];
		If[report===True,
		Print["    = ",u];
		Print["    = ",u /. Map[(#[[1]] -> ToString[#[[2]]])&,list]];
		Print["    = ",u /. rules],];
		res = {var,expr /. rules, u /. rules};
		If[rounding===True,sr = SRound[res];
        	If[report===True,Print["Runden der Ergebnisse:  ",resdis," = ",res[[3]],
                 " \[TildeTilde] ",sr[[3]],"  \[DoubleRightArrow]  ",var," = ",
                 res[[2]]," \[TildeTilde] ",sr[[2]]]];sr,res]
		]


Regression[adata_List,afuns_List,avar_,aparam_] := 
  Module[{data=adata,funs=afuns,var=avar,param=aparam,res,p,i, indices, 
      constants, sol},
		res = Regress[data,funs,var,IncludeConstant -> False];
		p = (ParameterTable /. res)[[1]];
		indices = Table[i,{i,1,Length[p]}];
		constants = Map[param[#]&,indices];
		sol = Partition[
        Flatten[Transpose[{constants,
              Map[{#[[1]],#[[2]]}&,p]}]],3];
		{constants.funs,var,sol}]

PolarListPlot[apoints_List,opts___Rule] := 
  Module[{s,p,points=apoints,max,gap ,anglegap,i,aktpoints,hp,splines,tpoints,m},

		If[Depth[points]=== 4,aktpoints = points, aktpoints = {points};];		
		max = MaxValue /. {opts} /. MaxValue ->  Ceiling[Max[Flatten[Map[Transpose[#][[2]]&,aktpoints]]]];
    
		gap = ValueGap /. {opts} /. ValueGap -> max/5;
		anglegap = AngleGap /. {opts} /. AngleGap -> 30;
		splines = SplineInterpolation /. {opts} /. SplineInterpolation -> True;
		
		s = {};
		p = Flatten[Table[{Circle[{0,0},i,{0,180 Degree}],Text[ToString[N[i]],{i,0},{0,1}]},{i,gap,max,gap}]];
		p = Join[p,Flatten[Table[{Line[{{0,0},{Cos[i Degree]*max,Sin[i Degree]*max}}],
                 Text[ToString[i],{Cos[i Degree]*max*1.05,Sin[i Degree]*max*1.05}]},{i,0,180,anglegap}]]];
		hp = {};
		i =1 ;
                While[i<=Length[aktpoints],
		hp = Append[hp,GrayLevel[(i-1) 0.8/Length[aktpoints]]];
		hp = Append[hp,PointSize[0.02]];
		tpoints = 
                Map[{Cos[#[[1]] Degree]*#[[2]],Sin[#[[1]] Degree]*#[[2]]}&,aktpoints[[i]]];
		hp = Join[hp,Map[Point,tpoints]]; 
                If[(splines === True) && (Length[tpoints]>=3),s=Append[s,Graphics[Spline[tpoints,Cubic]]],];i++;];
		p = Append[p,hp];
		Show[Join[{Graphics[N[Flatten[p]]]},s],AspectRatio -> Automatic,PlotRange -> All] 
		]

End[ ]

Protect[SRound]
Protect[EvaluateSeries]
Protect[Gauss]
Protect[Regression]
Protect[Rounding]
Protect[Report]
Protect[MaxValue]
Protect[ValueGap]
Protect[AngleGap]
Protect[SplineInterpolation]
Protect[PolarListPlot]

EndPackage[ ]
